Fix carp not deleting addresses correctly, which breaks carp interfaces setup using the rc.d/jail.
--- sys/netinet/in.c.orig	2013-02-19 17:06:48.000000000 +0000
+++ sys/netinet/in.c	2013-02-26 14:52:53.293628430 +0000
@@ -583,6 +583,20 @@
 
 	case SIOCDIFADDR:
 		/*
+		 * Carp and possibly others expect their if_ioctl handler
+		 * called for SIOCDIFADDR
+		 */
+		if (ifp->if_ioctl != NULL) {
+			error = (*ifp->if_ioctl)(ifp, SIOCDIFADDR, (caddr_t)ia);
+			/*
+			 * We ignore EINVAL and ENOTTY as this call is likely not
+			 * supported by most drivers
+			 */
+			if (error && error != EINVAL && error != ENOTTY)
+				goto out;
+		}
+
+		/*
 		 * in_ifscrub kills the interface route.
 		 */
 		in_ifscrub(ifp, ia, LLE_STATIC);
--- sys/netinet/ip_carp.c.orig	2013-02-25 19:03:18.801241179 +0000
+++ sys/netinet/ip_carp.c	2013-02-25 19:30:54.401573772 +0000
@@ -1628,21 +1628,17 @@
 
 	if (!--sc->sc_naddrs) {
 		struct carp_if *cif = (struct carp_if *)sc->sc_carpdev->if_carp;
-		struct ip_moptions *imo = &sc->sc_imo;
 
 		CARP_LOCK(cif);
-		callout_stop(&sc->sc_ad_tmo);
-		SC2IFP(sc)->if_flags &= ~IFF_UP;
-		SC2IFP(sc)->if_drv_flags &= ~IFF_DRV_RUNNING;
-		sc->sc_vhid = -1;
-		in_delmulti(imo->imo_membership[--imo->imo_num_memberships]);
-		imo->imo_multicast_ifp = NULL;
-		TAILQ_REMOVE(&cif->vhif_vrs, sc, sc_list);
-		if (!--cif->vhif_nvrs) {
-			sc->sc_carpdev->if_carp = NULL;
-			CARP_LOCK_DESTROY(cif);
-			free(cif, M_CARP);
+		if (cif->vhif_nvrs == 1) {
+			/* Last address so detach */
+			carpdetach(sc, 1);
 		} else {
+			/* Just the last IPv4 address */
+			callout_stop(&sc->sc_md_tmo);
+			carp_multicast_cleanup(sc, 1);
+			TAILQ_REMOVE(&cif->vhif_vrs, sc, sc_list);
+			cif->vhif_nvrs--;
 			CARP_UNLOCK(cif);
 		}
 	}
@@ -1836,18 +1831,17 @@
 		struct carp_if *cif = (struct carp_if *)sc->sc_carpdev->if_carp;
 
 		CARP_LOCK(cif);
-		callout_stop(&sc->sc_ad_tmo);
-		SC2IFP(sc)->if_flags &= ~IFF_UP;
-		SC2IFP(sc)->if_drv_flags &= ~IFF_DRV_RUNNING;
-		sc->sc_vhid = -1;
-		carp_multicast6_cleanup(sc, 1);
-		TAILQ_REMOVE(&cif->vhif_vrs, sc, sc_list);
-		if (!--cif->vhif_nvrs) {
-			CARP_LOCK_DESTROY(cif);
-			sc->sc_carpdev->if_carp = NULL;
-			free(cif, M_CARP);
-		} else
+		if (cif->vhif_nvrs == 1) {
+			/* Last address so detach */
+			carpdetach(sc, 1);
+		} else {
+			/* Just the last IPv6 address */
+			callout_stop(&sc->sc_md6_tmo);
+			carp_multicast6_cleanup(sc, 1);
+			TAILQ_REMOVE(&cif->vhif_vrs, sc, sc_list);
+			cif->vhif_nvrs--;
 			CARP_UNLOCK(cif);
+		}
 	}
 
 	return (error);
